grammar test;

options {
  language = Java;
  //output=AST;
  //ASTLabelType=CommonTree;
}

@header {
	package trans;
	import java.util.Vector;
	
}

@lexer::header {
	package trans;
}

model returns [Model model]
	: 'Model {' 'Name' ID {$model = new Model($ID.text.substring(1,$ID.text.length()-1));} 
	(~('System {'))* system {$model.addSystem($system.system);} '}' EOF
	;
	
system returns [Sim_System system] 
	:  'System {' 'Name' ID {$system = new Sim_System($ID.text.substring(1,$ID.text.length()-1));} .* 
	('Block {' 'BlockType' block {$system.addBlock($block.block);} '}')+ 
	(lineDeclaration {$lineDeclaration.line.define($system.block_array); $system.addLine($lineDeclaration.line);} )+ 
	('Annotation {' .* '}')*
	'}' 
	;

block returns [Block block] 
	: 
	(constant {$block = new Constant(); $block=$constant.constant;} 
	|sum {$block = new Sum(); $block=$sum.sum;}
	|sscope {$block = new Scope(); $block=$sscope.scope;}
	|gain {$block = new Gain(); $block=$gain.gain;}
	|discretePulseGenerator {$block = new DiscretePulseGenerator(); $block=$discretePulseGenerator.gener;}
	|unitDelay {$block = new UnitDelay(); $block=$unitDelay.delay;}
	|discreteIntegrator {$block = new DiscreteIntegrator(); $block=$discreteIntegrator.integrator;}
	|subsystem {$block=new SubSystem(); $block=$subsystem.subsystem; $subsystem.subsystem.s_system.sys_name=$subsystem.subsystem.block_name; $subsystem.subsystem.s_system.is_sub=true;}
	|inPort {$block= new InPort(); $block=$inPort.port;}
	|triggerPort {$block = new TriggerPort(); $block=$triggerPort.trig_port;}
	|outPort {$block = new OutPort(); $block=$outPort.out;}
	|zeroOrderHold {$block = new ZeroOrderHold(); $block=$zeroOrderHold.zero_hold;}
	|rateTransition {$block=new OutPort(); $block=$rateTransition.out;}
	|enablePort {$block= new EnablePort(); $block=$enablePort.en_port;} 
	|product {$block= new Product(); $block=$product.product;}
	|logic {$block= new Logic(); $block=$logic.logic;}
	|transferFcn {$block = new TransferFcn(); $block=$transferFcn.trans;}
	|lookup {$block = new Lookup(); $block=$lookup.look;}
	|signum {$block = new Signum(); $block=$signum.sign;}
	|saturation {$block = new Saturation(); $block=$saturation.sat;}
	|reference {$block = new Reference(); $block=$reference.ref;}
	|fcn {$block = new Fcn(); $block=$fcn.fcn;}
	|relational {$block = new Relational(); $block=$relational.rel;}
	|terminator {$block = new Terminator(); $block=$terminator.terminator;}
	|sine {$block = new Sine(); $block=$sine.sine;}
	|convertion {$block = new Convertion(); $block=$convertion.conv;}
	|abs {$block = new Abs(); $block=$abs.abs;}
	|relay {$block = new Relay(); $block=$relay.relay;}
	|switc {$block= new Switch(); $block=$switc.swit;})
	;


lineDeclaration returns [Line line]	
	: {$line = new Line();}
	'Line {' ('Name' ID)? ('Points' .*)? ('Labels' .*)? srcBlock {$line.src_block=$srcBlock.block;}  
	srcPort {$line.src_port=$srcPort.port;} .* ('Points' .*)? 
	((b_branch { $line.branch_array.addAll(new Vector<Branch>($b_branch.v_branch));	})+
	|dstBlock {$line.dst_block=$dstBlock.block;}
	dstPort {$line.dst_port=$dstPort.port;}) '}'
	;

b_branch returns [Vector <Branch> v_branch]
	: {$v_branch = new Vector<Branch>();}
	'Branch {' ('Labels' .*)? ('Points' .*)? 
	(branch { $v_branch.add(new Branch($branch.branch));}
	|c_branch {$v_branch.addAll(new Vector<Branch>($c_branch.c_branch));})+ '}'
	;

c_branch returns [Vector <Branch> c_branch]
	: {$c_branch = new Vector<Branch>();}
	'Branch {' ('Labels' .*)? ('Points' .*)? 
	(branch { $c_branch.add(new Branch($branch.branch));}
	|b_branch {$c_branch.addAll(new Vector<Branch>($b_branch.v_branch));})+ '}'
	;
	
branch returns [Branch branch]
	: {$branch =new Branch();}
	dstBlock {$branch.dst_block=$dstBlock.block;}
	dstPort {$branch.dst_port=$dstPort.port;}
	;
	
srcBlock returns [String block]
	: 'SrcBlock' ID {$block=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
srcPort returns [int port]
	: 'SrcPort' INT {$port=Integer.parseInt($INT.text);}
	;
	
dstBlock returns [String block]
	: 'DstBlock' ID {$block=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
dstPort returns [String port]
	: 'DstPort' (INT {$port=$INT.text;}
	|id1='trigger' {$port=$id1.text;} 
	|id2='enable' {$port=$id2.text;} )
	;
	
sum returns [Sum sum]
	: {$sum = new Sum();}
	'Sum' 'Name' ID {$sum.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	ports {$sum.ports=$ports.port.get(0);} .* 
	(inputs {$sum.sign=$inputs.sign;}.* )?
	(outDataType {$sum.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$sum.sample_time=$sampleTime.time;} .*)?
	;

inputs returns [String sign]
	: 'Inputs' '"' (op {$sign=$op.op;}| INT {$sign=$INT.text;}) '"'
	;

ports returns [Vector<Integer> port]
	: {$port=new Vector<Integer>();}
	'Ports' matrix {$port=$matrix.numb;}
	;
	
outDataType returns [String out_data_type]
	: 'OutDataType\t' ID {$out_data_type=$ID.text.substring(1,$ID.text.length()-1);}
	;


constant returns [Constant constant]
	: {$constant = new Constant(); boolean flag=true;}
	'Constant' 'Name' ID {$constant.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	(value {$constant.value=$value.a; flag=false;} .*)? {if(flag) $constant.value="1";}
	(outDataType {$constant.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$constant.sample_time=$sampleTime.time;} .*)? 
	;

value returns [String a]
	: 'Value' '"'(INT {$a=$INT.text;} | FLOAT {$a=$FLOAT.text;}) '"'//INT {$a=Integer.parseInt($INT.text);} '"'
	;

sscope returns [Scope scope]
	: {$scope = new Scope();}
	'Scope' 'Name' ID {$scope.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	ports {$scope.ports=$ports.port.get(0);} .* 'List {' .* '}' .*
	;
	
gain returns [Gain gain]
	: {$gain= new Gain();}
	'Gain' 'Name' ID {$gain.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	//.* (gainValue { String gainv=null; $gain.gain_value=$gainValue.value; $gain.gv=gainv;} .*)? 
	.* ('Gain' '"'(INT {$gain.gain_value=Double.parseDouble($INT.text);$gain.gv=$INT.text;} | FLOAT {$gain.gain_value=Double.parseDouble($FLOAT.text); $gain.gv=$FLOAT.text; })'"' .*)?
	(outDataType {$gain.out_data_type=$outDataType.out_data_type;} .*)? 
	(sampleTime {$gain.sample_time=$sampleTime.time;} .*)?
	('Port {' .* '}' .*)?
	;
	
//gainValue returns [double value]
//	: 'Gain' '"'(INT {$value=Double.parseDouble($INT.text);} | FLOAT {$value=Double.parseDouble($FLOAT.text); gainv=$FLOAT.text; })'"'// {$value=$num.num;} '"'//INT {$value=Integer.parseInt($INT.text);} '"'
//	;
	
discretePulseGenerator returns [DiscretePulseGenerator gener]
	: {$gener=new DiscretePulseGenerator();}
	'DiscretePulseGenerator' 'Name' ID {$gener.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	(amplitude {$gener.amplitude=$amplitude.amp;} .*)? 
	(period {$gener.period=$period.per;}.*)?
	(pulseWidth {$gener.pulse_width=$pulseWidth.pwidth;}.*)? 
	(sampleTime {$gener.sample_time=$sampleTime.time;}.*)?
	;
	
amplitude returns [double amp]
	: 'Amplitude' '"' num {$amp=$num.num;} '"'//INT {$amp=Integer.parseInt($INT.text);} '"'
	;

period returns [double per]
	:	'Period' '"' num {$per=$num.num;} '"'//INT {$per=Integer.parseInt($INT.text);} '"'
	;

pulseWidth returns [double pwidth]
	: 'PulseWidth' '"' num {$pwidth=$num.num;} '"'//INT {$pwidth=Integer.parseInt($INT.text);} '"'
	;
	
sampleTime returns [double time]
	: 'SampleTime' '"' num {$time=$num.num;} '"'//INT {$time=Integer.parseInt($INT.text);} '"'
	;

unitDelay returns [UnitDelay delay]
	: {$delay=new UnitDelay();} 
	'UnitDelay' 'Name' ID {$delay.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	('X0' '"'num {$delay.initial=$num.num; } '"' .*)?
	(sampleTime {$delay.sample_time=$sampleTime.time;})?
	;
	
discreteIntegrator returns [DiscreteIntegrator integrator]
	: {$integrator=new DiscreteIntegrator();}
	'DiscreteIntegrator' 'Name' ID {$integrator.block_name=$ID.text.substring(1,$ID.text.length()-1);} 
	ports {$integrator.ports=$ports.port.get(0);} .* 
	integratorMeth {$integrator.int_method=$integratorMeth.int_method;}
	externalReset {$integrator.ext_reset=$externalReset.ext_reset;} .* 
	initialCond {$integrator.init_cond=$initialCond.init_cond;} 
	sampleTime {$integrator.sample_time=$sampleTime.time;} .* 
	(outDataType {$integrator.out_data_type=$outDataType.out_data_type;} .*)?
	;
	
externalReset returns [String ext_reset]
	: 'ExternalReset' ID {$ext_reset=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
integratorMeth returns [String int_method]
	: 'IntegratorMethod' ID {$int_method=$ID.text.substring(1,$ID.text.length()-1);}
	;

initialCond returns [double init_cond]
	: 'InitialCondition	      ' '"' num {$init_cond=$num.num;} '"' //INT {$init_cond=Integer.parseInt($INT.text);} '"'
	;
	
subsystem returns [SubSystem subsystem]
	: {$subsystem=new SubSystem();} 
	'SubSystem' 'Name' ID {$subsystem.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	ports {$subsystem.ports=$ports.port;} .* 
	system {$subsystem.s_system=$system.system;}
    ;
      	
inPort returns [InPort port]
	: {$port= new InPort();}
	'Inport' 'Name' ID {$port.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	(('Port' .* '"'INT {$port.port=Integer.parseInt($INT.text);} '"'.* ))?
	(outDataType {$port.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$port.sample_time=$sampleTime.time;} .*)?
	('Port {' .* '}')?
      ;

triggerPort returns [TriggerPort trig_port]
	: {$trig_port=new TriggerPort();}
	'TriggerPort' 'Name' ID {$trig_port.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	(triggerType {$trig_port.trig_type=$triggerType.type;} .*)? 
	(statesWhenEnabling {$trig_port.states_enabl=$statesWhenEnabling.states;} .*)?
     ;
      
triggerType returns [String type]
	: 'TriggerType' ID {$type=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
statesWhenEnabling returns [String states]	
	: 'StatesWhenEnabling' ID {$states=$ID.text.substring(1,$ID.text.length()-1);}
	;

outPort returns [OutPort out]
	: {$out=new OutPort();} 
	'Outport' 'Name' e=ID {$out.block_name=$e.text.substring(1,$e.text.length()-1);} .* 
	('Port' .* '"'INT {$out.port=Integer.parseInt($INT.text);} '"' .*)?
	(outDataType {$out.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$out.sample_time=$sampleTime.time;} .*)?
	('OutputWhenDisabled' e1=ID {$out.disable=$e1.text.substring(1,$e1.text.length()-1);})?
	('InitialOutput' '"' num {$out.initial=$num.num; }'"' .*)?
    ;
    
rateTransition returns [OutPort out] : {$out=new OutPort();} 'RateTransition' 'Name' ID {$out.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* ;

product returns [Product product]
	: {$product=new Product();}
	'Product' 'Name' ID {$product.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	ports {$product.ports=$ports.port.get(0);} .* 
	(inputs {$product.operator=$inputs.sign;}.*)?
	(outDataType {$product.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$product.sample_time=$sampleTime.time;} .*)?
	;
	
switc returns [Switch swit]
	: {$swit=new Switch();}
	'Switch' 'Name' ID {$swit.block_name=$ID.text.substring(1,$ID.text.length()-1);} .*
	(criteria {$swit.criteria=$criteria.criteria; } .*)?
	(threshold {$swit.threshold=$threshold.thres;} .*)? 
	(outDataType {$swit.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$swit.sample_time=$sampleTime.time;} .*)?
	;

logic returns [Logic logic]
	: {$logic=new Logic();}
	'Logic' 'Name' ID {$logic.block_name=$ID.text.substring(1,$ID.text.length()-1);}
	ports {$logic.ports=$ports.port.get(0);} .* 
	(operator {$logic.operator=$operator.operator;} .*)?
	//(outDataType {$product.out_data_type=$outDataType.out_data_type;} .*)?
	(sampleTime {$logic.sample_time=$sampleTime.time;} .*)?
	;
 
 operator returns [String operator]
 	: 'Operator' ID {$operator=$ID.text.substring(1,$ID.text.length()-1);}
 	;
 
criteria returns [String criteria ]
	: 'Criteria' ID {$criteria=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
threshold returns [double thres]
	: 'Threshold' '"' num {$thres=$num.num;} '"'//INT {$thres=Integer.parseInt($INT.text);} '"'
	;
 
zeroOrderHold returns [ZeroOrderHold zero_hold]
	: {$zero_hold=new ZeroOrderHold();}
	'ZeroOrderHold' 'Name' ID {$zero_hold.block_name=$ID.text.substring(1,$ID.text.length()-1);} .* 
	sampleTime {$zero_hold.sample_time=$sampleTime.time;}
	;
	
enablePort returns [EnablePort en_port]
	: {$en_port = new EnablePort();}
	'EnablePort' 'Name' e1=ID {$en_port.block_name=$e1.text.substring(1,$e1.text.length()-1);} .*
	('StatesWhenEnabling' e=ID {$en_port.states=$e.text.substring(1,$e.text.length()-1);} .*)?
	;
	
numerator returns [Vector<Double> numerator]
	: {$numerator = new Vector<Double>();}
	'Numerator' '"' '[' (num {Double temp=new Double($num.num); $numerator.add(temp);} )+ ']' '"'
	;

denominator returns [Vector<Double> denominator]
	:{$denominator = new Vector<Double>();} 
	'Denominator' '"' '[' (num {Double temp=new Double($num.num); $denominator.add(temp);} )+ ']' '"'
	;

transferFcn returns [TransferFcn trans]
	: {$trans= new TransferFcn();}
	('TransferFcn' | 'DiscreteFilter'|'DiscreteTransferFcn') 'Name' ID {$trans.block_name=$ID.text.substring(1,$ID.text.length()-1);} .*
	numerator { $trans.numerator=$numerator.numerator; } .*
	denominator { $trans.denominator=$denominator.denominator; } .*
	(sampleTime {$trans.sample_time=$sampleTime.time;} .*)?
	;
	
lookup returns [Lookup look ]
	: {$look = new Lookup();}
	'Lookup' 'Name' ID {$look.block_name=$ID.text.substring(1,$ID.text.length()-1);} .*
	inputValues { $look.input_values=$inputValues.values; } .*
	table { $look.table=$table.table; } .*
	(sampleTime {$look.sample_time=$sampleTime.time;} .*)?
	;
	
inputValues returns [Vector<Double> values]
	: {$values= new Vector<Double>();}
	'InputValues' '"' '[' (e=num {Double temp=new Double($num.num); $values.add(temp);})+ ']' '"'
	;

table returns [Vector<Double> table]
	: {$table= new Vector<Double>();}
	'Table' '"' '[' (e=num {Double temp=new Double($num.num); $table.add(temp);})+ ']' '"'
	;
	
signum returns [Signum sign]
	: {$sign = new Signum();}
	'Signum' 'Name' ID {$sign.block_name= $ID.text.substring(1,$ID.text.length()-1);} .* 
	(sampleTime {$sign.sample_time=$sampleTime.time;} .*)?
	;
	
convertion returns [Convertion conv]
	: {$conv = new Convertion();}
	'DataTypeConversion' 'Name' ID {$conv.block_name= $ID.text.substring(1,$ID.text.length()-1);} .*
	(sampleTime {$conv.sample_time=$sampleTime.time;} .*)?
	;
	
abs returns [Abs abs]
	:{ $abs = new Abs(); }
	'Abs''Name' ID {$abs.block_name= $ID.text.substring(1,$ID.text.length()-1);} .*
	(sampleTime {$abs.sample_time=$sampleTime.time;} .*)?
	;
	
saturation returns [Saturation sat]
	: {$sat = new Saturation();}
	'Saturate' 'Name' ID {$sat.block_name= $ID.text.substring(1,$ID.text.length()-1);} .*
	upperLimit {$sat.upper = $upperLimit.upp;} .*
	lowerLimit {$sat.lower = $lowerLimit.low;} .*
	(sampleTime {$sat.sample_time=$sampleTime.time;} .*)?
	;
	
sine returns [Sine sine]
	: {$sine = new Sine();}
	'Sin' 'Name' ID {$sine.block_name= $ID.text.substring(1,$ID.text.length()-1);} .*
	(amplitude {$sine.amplitude=$amplitude.amp;} .*)?
	(samples {$sine.samples=$samples.samples;} .*)?
	(offset {$sine.offset=$offset.offset;} .*)?
	(sampleTime {$sine.sample_time=$sampleTime.time;} .*)?
	;

relay returns [Relay relay]
	: {$relay = new Relay();}
	'Relay' 'Name' ID {$relay.block_name= $ID.text.substring(1,$ID.text.length()-1);} .*
	(onswitch {$relay.on_switch=$onswitch.val;} .*)?
	(offswitch {$relay.off_switch=$offswitch.val;} .*)?
	(onvalue {$relay.on_value=$onvalue.val;} .*)?
	(offvalue {$relay.off_value=$offvalue.val;} .*)?
	;

onswitch returns [double val]
	: 'OnSwitchValue' '"' num {$val=$num.num;} '"'
	;

offswitch returns [double val]
	: 'OffSwitchValue' '"' num {$val=$num.num;} '"'
	;

onvalue returns [double val]
	: 'OnOutputValue' '"' num {$val=$num.num;} '"'
	;
	
offvalue returns [double val]
	: 'OffOutputValue' '"' num {$val=$num.num;} '"'
	;
	
samples returns [int samples]
	: 'Samples' '"' INT {$samples=Integer.parseInt($INT.text);} '"'
	;

offset returns [int offset]
	: 'Offset' '"' INT {$offset=Integer.parseInt($INT.text);} '"'
	;
	
lowerLimit returns [double low]
	: 'LowerLimit' '"' num {$low=$num.num;} '"' //INT {$low=Integer.parseInt($INT.text);} '"'
	;

upperLimit returns [double upp]
	: 'UpperLimit' '"' num {$upp=$num.num;} '"' //INT {$upp=Integer.parseInt($INT.text);} '"'
	;
	
reference returns [Reference ref]
	: {$ref=new Reference();} 
	'Reference' 'Name' e=ID {$ref.block_name= $e.text.substring(1,$e.text.length()-1);} .*
	'relop' '"' op {$ref.operator=$op.op;} '"' .*
	;
	
fcn returns [Fcn fcn]
	:{$fcn=new Fcn();}
	'Fcn' 'Name' e=ID {$fcn.block_name= $e.text.substring(1,$e.text.length()-1);} .*
	expr {$fcn.expr=$expr.expr; }.*
	;
	
expr returns[String expr]
	: 'Expr' ID {$expr.expr=$ID.text.substring(1,$ID.text.length()-1);}
	;
	
relational returns[Relational rel]
	:{$rel = new Relational();}
	'RelationalOperator' 'Name' e=ID {$rel.block_name= $e.text.substring(1,$e.text.length()-1);} .*
	('Operator' '"' op {$rel.op=$op.op;} '"' .*)?
	(sampleTime {$rel.sample_time=$sampleTime.time;} .*)?
	;
	
terminator returns [Terminator terminator]
	: {$terminator = new Terminator(); }
	'Terminator' 'Name' e=ID {$terminator.block_name= $e.text.substring(1,$e.text.length()-1);} .*
	;
	
op returns[String op]
	:
	e='==' {$op=$e.text;}
	| e1='<=' {$op=$e1.text;}
	| e2='<' {$op=$e2.text;}
	| e3='>=' {$op=$e3.text;}
	| e4='>' {$op=$e4.text;}
	| e5='~=' {$op=$e5.text;}
	| e6='++' {$op=$e6.text;}
	| e7='+-' {$op=$e7.text;}
	| e8='-+' {$op=$e8.text;}
	| e15='|+-' {$op="+-";}
	| e10='/*' {$op="dp";}
	| e11='*/' {$op="pd";}
	| e9='--' {$op=$e9.text;}
	| e12='|++' {$op="++";}
	| e13='+-|' {$op="+-";}
	;

ID 	: '"' ('a'..'z'|'A'..'Z'|'_'|'|') //('0'..'9')* '"'
		('a'..'z'|'A'..'Z'|'_'|'-'|'.'|'+'|'\\'|'<'|'>'|'='|'&'|'~'|' '|':'|'('|')'|'0'..'9')* '"';

WS 	:   (' '|'\r'|'\t'|'\u000C'|'\n') {$channel=HIDDEN;};

matrix returns [Vector<Integer> numb]
	:{$numb= new Vector<Integer>();}
	'[' e=INT { Integer temp=new Integer(Integer.parseInt($e.text));
	$numb.add(temp);}
	//$numb=Integer.parseInt($e.text);}
	(','e2=INT {Integer temp2=new Integer(Integer.parseInt($e2.text));
	$numb.add(temp2);})* ']';

num returns[double num]
	:INT {$num=Double.parseDouble($INT.text);} | FLOAT {$num=Double.parseDouble($FLOAT.text);}
	;

INT : '-'?('0'..'9')+ ;//('.'('0'..'9'))*;

FLOAT: '-'?('0'..'9')+ '.'('0'..'9')+
	|	'-'?('0'..'9')+ '.'('0'..'9')+ 'e' ('-'|'+')('0'..'9')+
	;

